@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: device.c,v 1.21 2006/01/21 06:36:51 david Exp $
+ * CVSID: $Id$
  *
  * device.c : HalDevice methods
  *
@@ -35,6 +35,7 @@
 #include "device.h"
 #include "hald_marshal.h"
 #include "logger.h"
+#include "hald_runner.h"
 
 static GObjectClass *parent_class;
 
@@ -57,11 +58,14 @@ hal_device_finalize (GObject *obj)
 {
 	HalDevice *device = HAL_DEVICE (obj);
 
+	runner_device_finalized (device);
+
 #ifdef HALD_MEMLEAK_DBG
 	dbg_hal_device_object_delta--;
 	printf ("************* in finalize for udi=%s\n", device->udi);
 #endif
 
+
 	g_slist_foreach (device->properties, (GFunc) hal_property_free, NULL);
 
 	g_free (device->udi);
@@ -132,6 +136,8 @@ hal_device_init (HalDevice *device)
 
 	device->udi = g_strdup_printf ("/org/freedesktop/Hal/devices/temp/%d",
 				       temp_device_counter++);
+	device->num_addons = 0;
+	device->num_addons_ready = 0;
 }
 
 GType
@@ -567,7 +573,7 @@ hal_device_property_get_as_string (HalDevice *device, const char *key, char *buf
 			snprintf (buf, bufsize, "%d", hal_property_get_int (prop));
 			break;
 		case HAL_PROPERTY_TYPE_UINT64:
-			snprintf (buf, bufsize, "%lld", hal_property_get_uint64 (prop));
+			snprintf (buf, bufsize, "%llu", (long long unsigned int) hal_property_get_uint64 (prop));
 			break;
 		case HAL_PROPERTY_TYPE_DOUBLE:
 			snprintf (buf, bufsize, "%f", hal_property_get_double (prop));
@@ -946,9 +952,9 @@ hal_device_print (HalDevice *device)
                         break;
  
                 case HAL_PROPERTY_TYPE_UINT64:
-                        fprintf (stderr, "  %s = %lld  0x%llx  (uint64)\n", key,
-                                hal_property_get_uint64 (p),
-                                hal_property_get_uint64 (p));
+                        fprintf (stderr, "  %s = %llu  0x%llx  (uint64)\n", key,
+                                (long long unsigned int) hal_property_get_uint64 (p),
+                                (long long unsigned int) hal_property_get_uint64 (p));
                         break;
  
                 case HAL_PROPERTY_TYPE_DOUBLE:
@@ -1294,3 +1300,51 @@ hal_device_property_strlist_remove (HalDevice *device,
 	
 	return TRUE;
 }
+
+gboolean
+hal_device_property_strlist_is_empty (HalDevice    *device,
+				      const char   *key)
+{
+	GSList *strlist;
+
+	if ( hal_device_has_property (device, key)) {
+		strlist = hal_device_property_get_strlist (device, key);
+		if (strlist == NULL ) 
+			return TRUE;
+
+		if (g_slist_length (strlist) > 0) 
+			return FALSE;
+		else 
+			return TRUE;
+	}
+	return FALSE;
+}
+
+void
+hal_device_inc_num_addons (HalDevice *device)
+{
+	device->num_addons++;
+}
+
+gboolean
+hal_device_inc_num_ready_addons (HalDevice *device)
+{
+	if (hal_device_are_all_addons_ready (device)) {
+		HAL_ERROR (("In hal_device_inc_num_ready_addons for udi=%s but all addons are already ready!", 
+			    device->udi));
+		return FALSE;
+	}
+
+	device->num_addons_ready++;
+	return TRUE;
+}
+
+gboolean
+hal_device_are_all_addons_ready (HalDevice *device)
+{
+	if (device->num_addons_ready == device->num_addons) {
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
