@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: device.h,v 1.14 2006/01/21 06:36:51 david Exp $
+ * CVSID: $Id$
  *
  * device.c : HalDevice methods
  *
@@ -41,6 +41,9 @@ struct _HalDevice {
 	char *udi;
 	
 	GSList *properties;
+
+	int num_addons;
+	int num_addons_ready;
 };
 
 struct _HalDeviceClass {
@@ -173,7 +176,8 @@ gboolean      hal_device_property_strlist_add (HalDevice    *device,
 gboolean      hal_device_property_strlist_remove (HalDevice    *device,
 						  const char   *key,
 						  const char *value);
-
+gboolean      hal_device_property_strlist_is_empty (HalDevice    *device,
+                                                    const char   *key);
 
 gboolean      hal_device_property_remove     (HalDevice    *device,
 					      const char   *key);
@@ -201,4 +205,11 @@ gboolean      hal_device_property_set_attribute (HalDevice *device,
 						 enum PropertyAttribute attr,
 						 gboolean persistence);
 
+void          hal_device_inc_num_addons (HalDevice *device);
+
+gboolean      hal_device_inc_num_ready_addons (HalDevice *device);
+
+gboolean      hal_device_are_all_addons_ready (HalDevice *device);
+
+
 #endif /* DEVICE_H */
