@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: device_info.c,v 1.32 2006/01/21 06:36:51 david Exp $
+ * CVSID: $Id$
  *
  * device_store.c : Search for .fdi files and merge on match
  *
@@ -41,6 +41,7 @@
 #include "logger.h"
 #include "device_info.h"
 #include "device_store.h"
+#include "util.h"
 
 /**
  * @defgroup DeviceInfo Device Info File Parsing
@@ -86,7 +87,10 @@ enum {
 	CURELEM_REMOVE = 6,
 
 	/** Processing a clear element */
-	CURELEM_CLEAR = 7
+	CURELEM_CLEAR = 7,
+
+	/** Processing a spawn element */
+	CURELEM_SPAWN = 8
 };
 
 /** What and how to merge */
@@ -100,7 +104,8 @@ enum {
 	MERGE_TYPE_COPY_PROPERTY = 6,
 	MERGE_TYPE_STRLIST       = 7,
 	MERGE_TYPE_REMOVE        = 8,
-	MERGE_TYPE_CLEAR         = 9
+	MERGE_TYPE_CLEAR         = 9,
+	MERGE_TYPE_SPAWN         = 10
 };
 
 /** Parsing Context
@@ -455,19 +460,32 @@ handle_match (ParsingContext * pc, const char **attr)
 				return TRUE;
 		}
 	} else if (strcmp (attr[2], "empty") == 0) {
+		int type;
 		dbus_bool_t is_empty = TRUE;
 		dbus_bool_t should_be_empty = TRUE;
 
+
 		if (strcmp (attr[3], "false") == 0)
 			should_be_empty = FALSE;
 
-		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_STRING)
+		type = hal_device_property_get_type (d, prop_to_check);
+		switch (type) {
+		case HAL_PROPERTY_TYPE_STRING: 
+			if (hal_device_has_property (d, prop_to_check))
+				if (strlen (hal_device_property_get_string (d, prop_to_check)) > 0)
+					is_empty = FALSE;
+			break;
+		case HAL_PROPERTY_TYPE_STRLIST:
+			if (hal_device_has_property (d, prop_to_check))
+				if (!hal_device_property_strlist_is_empty(d, prop_to_check))
+					is_empty = FALSE;
+			break;
+		default:
+			/* explicit fallthrough */
 			return FALSE;
-
-		if (hal_device_has_property (d, prop_to_check))
-			if (strlen (hal_device_property_get_string (d, prop_to_check)) > 0)
-				is_empty = FALSE;
-
+			break;
+		} 
+	
 		if (should_be_empty) {
 			if (is_empty)
 				return TRUE;
@@ -748,6 +766,35 @@ handle_append_prepend (ParsingContext * pc, const char **attr)
 	return;
 }
 
+
+/** Called when the spawn element begins.
+ *
+ *  @param  pc                  Parsing context
+ *  @param  attr                Attribute key/value pairs
+ */
+static void
+handle_spawn (ParsingContext * pc, const char **attr)
+{
+	int num_attrib;
+
+	pc->merge_type = MERGE_TYPE_UNKNOWN;
+
+	for (num_attrib = 0; attr[num_attrib] != NULL; num_attrib++) {
+		;
+	}
+
+	if (num_attrib != 2)
+		return;
+
+	if (strcmp (attr[0], "udi") != 0)
+		return;
+
+	strncpy (pc->merge_key, attr[1], MAX_KEY_SIZE);
+
+	pc->merge_type = MERGE_TYPE_SPAWN;
+	return;
+}
+
 /** Called when the remove element begins.
  *
  *  @param  pc                  Parsing context
@@ -849,6 +896,8 @@ start (ParsingContext * pc, const char *el, const char **attr)
 
 	pc->cdata_buf_len = 0;
 
+	pc->merge_type = MERGE_TYPE_UNKNOWN;
+
 /*
     for (i = 0; i < pc->depth; i++)
         printf("  ");
@@ -988,6 +1037,21 @@ start (ParsingContext * pc, const char *el, const char **attr)
 			parsing_abort (pc);
 		}
 		pc->curelem = CURELEM_DEVICE_INFO;
+	} else if (strcmp (el, "spawn") == 0) {
+		if (pc->curelem != CURELEM_MATCH) {
+			HAL_ERROR (("%s:%d:%d: Element <spawn> can only be "
+				    "inside <match>", 
+				    pc->file, 
+				    XML_GetCurrentLineNumber (pc->parser), 
+				    XML_GetCurrentColumnNumber (pc->parser)));
+			parsing_abort (pc);
+		}
+
+		pc->curelem = CURELEM_SPAWN;
+		if (pc->match_ok) {
+			handle_spawn (pc, attr);
+		} 
+
 	} else {
 		HAL_ERROR (("%s:%d:%d: Unknown element <%s>",
 			    pc->file,
@@ -1006,6 +1070,17 @@ start (ParsingContext * pc, const char *el, const char **attr)
 
 }
 
+static void 
+spawned_device_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer userdata2)
+{
+	HAL_INFO (("Add callouts completed udi=%s", d->udi));
+
+	/* Move from temporary to global device store */
+	hal_device_store_remove (hald_get_tdl (), d);
+	hal_device_store_add (hald_get_gdl (), d);
+
+}
+
 /** Called by expat when an element ends.
  *
  *  @param  pc                  Parsing context
@@ -1201,6 +1276,31 @@ end (ParsingContext * pc, const char *el)
 				hal_device_property_remove (pc->device, pc->merge_key);
 			}
 		}
+	} else if (pc->merge_type == MERGE_TYPE_SPAWN) {
+		HalDevice *spawned;
+
+		spawned = hal_device_store_find (hald_get_gdl (), pc->merge_key);
+		if (spawned == NULL)
+			spawned = hal_device_store_find (hald_get_tdl (), pc->merge_key);
+
+		if (spawned == NULL) {
+			HAL_INFO (("Spawning new device object '%s' caused by <spawn> on udi '%s'", 
+				   pc->merge_key, pc->device->udi));
+
+			spawned = hal_device_new ();
+			hal_device_property_set_string (spawned, "info.bus", "unknown");
+			hal_device_property_set_string (spawned, "info.udi", pc->merge_key);
+			hal_device_property_set_string (spawned, "info.parent", pc->device->udi);
+			hal_device_set_udi (spawned, pc->merge_key);
+			
+			hal_device_store_add (hald_get_tdl (), spawned);
+			
+			di_search_and_merge (spawned, DEVICE_INFO_TYPE_INFORMATION);
+			di_search_and_merge (spawned, DEVICE_INFO_TYPE_POLICY);
+			
+			hal_util_callout_device_add (spawned, spawned_device_callouts_add_done, NULL, NULL);
+		}
+
 	} else if (pc->curelem == CURELEM_CLEAR && pc->match_ok) {
 		if (pc->merge_type == MERGE_TYPE_CLEAR) {
 			hal_device_property_strlist_clear (pc->device, pc->merge_key);
@@ -1208,7 +1308,6 @@ end (ParsingContext * pc, const char *el)
 	}
 
 
-
 	pc->cdata_buf_len = 0;
 	pc->depth--;
 
@@ -1368,8 +1467,12 @@ out:
 
 
 
-static int 
+static int
+#ifdef __GLIBC__
 my_alphasort(const void *a, const void *b)
+#else
+my_alphasort(const struct dirent **a, const struct dirent **b)
+#endif
 {
 	return -alphasort (a, b);
 }
