@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: logger.h,v 1.11 2005/11/02 15:38:13 david Exp $
+ * CVSID: $Id$
  *
  * logger.h : Logging facility
  *
@@ -49,6 +49,8 @@ enum {
 void logger_setup (int priority, const char *file, int line, const char *function);
 
 void logger_emit (const char *format, ...);
+//void logger_forward_debug (const char *format, va_list args);
+void logger_forward_debug (const char *format, ...);
 
 void logger_enable (void);
 void logger_disable (void);
@@ -56,6 +58,11 @@ void logger_disable (void);
 void logger_enable_syslog (void);
 void logger_disable_syslog (void);
 
+void setup_logger (void);
+
+#ifdef __SUNPRO_C
+#define __FUNCTION__ __func__
+#endif
 
 /** Trace logging macro */
 #define HAL_TRACE(expr)   do {logger_setup(HAL_LOGPRI_TRACE,   __FILE__, __LINE__, __FUNCTION__); logger_emit expr; } while(0)
