@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: util.c,v 1.36 2006/01/23 12:15:50 kay Exp $
+ * CVSID: $Id$
  *
  * util.c - Various utilities
  *
@@ -36,8 +36,10 @@
 #include <stdint.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <sys/wait.h>
+#include <sys/file.h>
 
 #include <glib.h>
 #include <dbus/dbus.h>
@@ -46,167 +48,11 @@
 #include "osspec.h"
 #include "logger.h"
 #include "hald.h"
+#include "hald_runner.h"
+#include "hald_dbus.h"
 #include "device_info.h"
 
-#include "hald_dbus.h"
 #include "util.h"
-#include "hald_runner.h"
-
-typedef struct {
-	int last_level;
-	int last_chargeRate;
-	time_t last_time;
-} batteryInfo;
-
-GHashTable *saved_battery_info = NULL;
-
-/** Given all the required parameters, this function will return the percentage
- *  charge remaining. There are lots of checks here as ACPI is often broken.
- *
- *  @param  id                  Optional ID given to this battery. Unused at present.
- *  @param  chargeLevel         The current charge level of the battery (typically mWh)
- *  @param  chargeLastFull      The last "full" charge of the battery (typically mWh)
- *  @return                     Percentage, -1 if invalid
- */
-int 
-util_compute_percentage_charge (const char *id,
-			     int chargeLevel,
-			     int chargeLastFull)
-{
-	int percentage;
-	/* make sure we have chargelevel */
-	if (chargeLevel <= 0) {
-		HAL_WARNING (("chargeLevel %i, returning -1!", chargeLevel));
-		return -1;
-	}
-	/* make sure not division by zero */
-	if (chargeLastFull > 0)
-		percentage = ((double) chargeLevel / (double) chargeLastFull) * 100;
-	else {
-		HAL_WARNING (("chargeLastFull %i, percentage returning -1!", chargeLastFull));
-		return -1;
-	}
-	/* Some bios's will report this higher than 100, limit it here */
-	if (percentage > 100) {
-		HAL_WARNING (("Percentage %i, returning 100!", percentage));
-		return 100;
-	}
-	/* Something really isn't right if we get a negative... */
-	if (percentage < 0) {
-		HAL_WARNING (("Percentage %i, returning -1!", percentage));
-		return -1;
-	}
-	return percentage;
-}
-
-/** Given all the required parameters, this function will return the number 
- *  of seconds until the battery is charged (if charging) or the number
- *  of seconds until empty (if discharging)
- *
- *  @param  id                  Optional ID given to this battery. Unused at present.
- *  @param  chargeRate          The "rate" (typically mW)
- *  @param  chargeLevel         The current charge level of the battery (typically mWh)
- *  @param  chargeLastFull      The last "full" charge of the battery (typically mWh)
- *  @param  isDischarging       If battery is discharging
- *  @param  isCharging          If battery is charging
- *  @param  guessChargeRate     If ignore chargeRate and guess them.
- *  @return                     Number of seconds, or -1 if invalid
- */
-int 
-util_compute_time_remaining (const char *id,
-			     int chargeRate,
-			     int chargeLevel,
-			     int chargeLastFull,
-			     gboolean isDischarging,
-			     gboolean isCharging,
-			     gboolean guessChargeRate)
-{
-	int remaining_time = 0;
-
-	/* should not get negative values */
-	if (chargeRate < 0 || chargeLevel < 0 || chargeLastFull < 0) {
-		HAL_WARNING (("chargeRate, chargeLevel or chargeLastFull < 0, returning -1"));
-		return -1;
-	}
-	/* batteries cannot charge and discharge at the same time */
-	if (isDischarging && isCharging) {
-		HAL_WARNING (("isDischarging & isCharging TRUE, returning -1"));
-		return -1;
-	}
-	/* 
-	 * Some laptops don't supply any rate info, but that's no reason for HAL not
-	 * to. We use the current and previous chargeLevel to estimate the rate.
-	 * The info is stored in a GHashTable because there could be more than one battery.
-	 */
-	if (chargeRate == 0 || guessChargeRate) {
-		batteryInfo *battery_info;
-		time_t cur_time = time(NULL);
-
-		/* Initialize the save_battery_info GHashTable */
-		if (!saved_battery_info) 
-			saved_battery_info = g_hash_table_new(g_str_hash, g_str_equal);
-
-		if ((battery_info = g_hash_table_lookup(saved_battery_info, id))) {
-			/* check this to prevent division by zero */
-			if ((cur_time == battery_info->last_time) || (chargeLevel == battery_info->last_level)) {
-				/* if we can't calculate because nothing changed, use last 
-				 * chargeRate to prevent removing battery.remaining_time.
-				 */
-				chargeRate = battery_info->last_chargeRate;
-			} else {
-				chargeRate = ((chargeLevel - battery_info->last_level) * 60 * 60) / (cur_time - battery_info->last_time);
-				/*
-				 * During discharging chargeRate would be negative, which would
-				 * mess up the the calculation below, so we make sure it's always
-				 * positive.
-				 */ 
-				chargeRate = (chargeRate > 0) ? chargeRate : -chargeRate;
-	
-				battery_info->last_level = chargeLevel;
-				battery_info->last_time = cur_time;
-				battery_info->last_chargeRate = chargeRate;
-			}
-		} else {
-			battery_info = g_new0(batteryInfo, 1);
-			g_hash_table_insert(saved_battery_info, (char*) id, battery_info);
-
-			battery_info->last_level = chargeLevel;
-			battery_info->last_time = cur_time;
-			battery_info->last_chargeRate = 0;
- 			return -1;
-		}
-	} 
-
-	if (chargeRate == 0)
-		return -1;
-
-	if (isDischarging) { 
-		remaining_time = ((double) chargeLevel / (double) chargeRate) * 60 * 60;
-	} else if (isCharging) {
-		/* 
-		 * Some ACPI BIOS's don't update chargeLastFull, 
-		 * so return 0 as we don't know how much more there is left
-		 */
-		if (chargeLevel > chargeLastFull ) {
-			HAL_WARNING (("chargeLevel > chargeLastFull, returning -1"));
-			return -1;
-		}
-		remaining_time = ((double) (chargeLastFull - chargeLevel) / (double) chargeRate) * 60 * 60;
-	}
-	
-	/* This shouldn't happen, but check for completeness */
-	if (remaining_time < 0) {
-		HAL_WARNING (("remaining_time %i, returning -1", remaining_time));
-		remaining_time = -1;
-	}
-	/* Battery life cannot be above 60 hours */
-	else if (remaining_time > 60*60*60) {
-		HAL_WARNING (("remaining_time *very* high, returning -1"));
-		remaining_time = -1;
-	}
-
-	return remaining_time;
-}
 
 gboolean 
 hal_util_remove_trailing_slash (gchar *path)
@@ -276,7 +122,6 @@ hal_util_get_parent_path (const gchar *path)
 gchar *
 hal_util_get_normalized_path (const gchar *path1, const gchar *path2)
 {
-	int i;
 	int len1;
 	int len2;
 	const gchar *p1;
@@ -284,18 +129,21 @@ hal_util_get_normalized_path (const gchar *path1, const gchar *path2)
 	gchar buf[HAL_PATH_MAX];
 
 	len1 = strlen (path1);
-	len2 = strlen (path1);
+	len2 = strlen (path2);
 
 	p1 = path1 + len1;
 
-	i = 0;
 	p2 = path2;
 	while (p2 < path2 + len2 && strncmp (p2, "../", 3) == 0) {
 		p2 += 3;
 
 		while (p1 >= path1 && *(--p1)!='/')
 			;
+	}
 
+	if ((p1-path1) < 0) {
+		HAL_ERROR (("Could not normalize '%s' and '%s', return 'NULL'", path1, path2)); 
+		return NULL;
 	}
 
 	strncpy (buf, path1, (p1-path1));
@@ -353,6 +201,57 @@ hal_util_set_int_from_file (HalDevice *d, const gchar *key, const gchar *directo
 	return ret;
 }
 
+
+gboolean
+hal_util_get_uint64_from_file (const gchar *directory, const gchar *file, guint64 *result, gint base)
+{
+	FILE *f;
+	char buf[64];
+	gchar path[HAL_PATH_MAX];
+	gboolean ret;
+
+	f = NULL;
+	ret = FALSE;
+
+	g_snprintf (path, sizeof (path), "%s/%s", directory, file);
+
+	f = fopen (path, "rb");
+	if (f == NULL) {
+		HAL_ERROR (("Cannot open '%s'", path));
+		goto out;
+	}
+
+	if (fgets (buf, sizeof (buf), f) == NULL) {
+		HAL_ERROR (("Cannot read from '%s'", path));
+		goto out;
+	}
+
+	/* TODO: handle error condition */
+	*result = strtoll (buf, NULL, base);
+
+	ret = TRUE;
+
+out:
+	if (f != NULL)
+		fclose (f);
+
+	return ret;
+}
+
+gboolean
+hal_util_set_uint64_from_file (HalDevice *d, const gchar *key, const gchar *directory, const gchar *file, gint base)
+{
+	guint64 value;
+	gboolean ret;
+
+	ret = FALSE;
+
+	if (hal_util_get_uint64_from_file (directory, file, &value, base))
+		ret = hal_device_property_set_uint64 (d, key, value);
+
+	return ret;
+}
+
 gboolean
 hal_util_get_bcd2_from_file (const gchar *directory, const gchar *file, gint *result)
 {
@@ -1017,16 +916,22 @@ hal_util_strdup_valid_utf8 (const char *str)
 {
 	char *endchar;
 	char *newstr;
+	unsigned int count = 0;
 
 	if (str == NULL)
 		return NULL;
 
 	newstr = g_strdup (str);
 
-	while (!g_utf8_validate (str, -1, (const char **) &endchar)) {
+	while (!g_utf8_validate (newstr, -1, (const char **) &endchar)) {
 		*endchar = '?';
+		count++;
 	}
-	return newstr;
+	
+	if (strlen(newstr) == count)
+		return NULL;
+	else
+		return newstr;
 }
 
 void
@@ -1071,3 +976,92 @@ hal_util_hexdump (const void *mem, unsigned int size)
 	}
 }
 
+gboolean
+hal_util_is_mounted_by_hald (const char *mount_point)
+{
+	int i;
+	FILE *hal_mtab;
+	int hal_mtab_len;
+	int num_read;
+	char *hal_mtab_buf;
+	char **lines;
+	gboolean found;
+
+	hal_mtab = NULL;
+	hal_mtab_buf = NULL;
+	found = FALSE;
+
+	/*HAL_DEBUG (("examining /media/.hal-mtab for %s", mount_point));*/
+
+	hal_mtab = fopen ("/media/.hal-mtab", "r");
+	if (hal_mtab == NULL) {
+		HAL_ERROR (("Cannot open /media/.hal-mtab"));
+		goto out;
+	}
+	if (fseek (hal_mtab, 0L, SEEK_END) != 0) {
+		HAL_ERROR (("Cannot seek to end of /media/.hal-mtab"));
+		goto out;
+	}
+	hal_mtab_len = ftell (hal_mtab);
+	if (hal_mtab_len < 0) {
+		HAL_ERROR (("Cannot determine size of /media/.hal-mtab"));
+		goto out;
+	}
+	rewind (hal_mtab);
+
+	hal_mtab_buf = g_new0 (char, hal_mtab_len + 1);
+	num_read = fread (hal_mtab_buf, 1, hal_mtab_len, hal_mtab);
+	if (num_read != hal_mtab_len) {
+		HAL_ERROR (("Cannot read from /media/.hal-mtab"));
+		goto out;
+	}
+	fclose (hal_mtab);
+	hal_mtab = NULL;
+
+	/*HAL_DEBUG (("hal_mtab = '%s'\n", hal_mtab_buf));*/
+
+	lines = g_strsplit (hal_mtab_buf, "\n", 0);
+	g_free (hal_mtab_buf);
+	hal_mtab_buf = NULL;
+
+	/* find the entry we're going to unmount */
+	for (i = 0; lines[i] != NULL && !found; i++) {
+		char **line_elements;
+
+		/*HAL_DEBUG ((" line = '%s'", lines[i]));*/
+
+		if ((lines[i])[0] == '#')
+			continue;
+
+		line_elements = g_strsplit (lines[i], "\t", 6);
+		if (g_strv_length (line_elements) == 6) {
+/*
+			HAL_DEBUG (("  devfile     = '%s'", line_elements[0]));
+			HAL_DEBUG (("  uid         = '%s'", line_elements[1]));
+			HAL_DEBUG (("  session id  = '%s'", line_elements[2]));
+			HAL_DEBUG (("  fs          = '%s'", line_elements[3]));
+			HAL_DEBUG (("  options     = '%s'", line_elements[4]));
+			HAL_DEBUG (("  mount_point = '%s'", line_elements[5]));
+			HAL_DEBUG (("  (comparing against '%s')", mount_point));
+*/
+
+			if (strcmp (line_elements[5], mount_point) == 0) {
+				found = TRUE;
+				/*HAL_INFO (("device at '%s' is indeed mounted by HAL's Mount()", mount_point));*/
+			}
+
+		}
+
+		g_strfreev (line_elements);
+	}
+
+out:
+	if (hal_mtab != NULL)
+		fclose (hal_mtab);
+	if (hal_mtab_buf != NULL)
+		g_free (hal_mtab_buf);
+
+	return found;
+}
+
+
