@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: util.h,v 1.17 2006/01/23 12:15:50 kay Exp $
+ * CVSID: $Id$
  *
  * util.h - Various utilities
  *
@@ -29,15 +29,18 @@
 #include "device.h"
 #include "device_store.h"
 
+#ifndef __FUNCTION__
+#define __FUNCTION__ __func__
+#endif
+
+#ifndef __GNUC__
+#define __attribute__(x)
+#endif
+
 #define HAL_NAME_MAX 256
-#define HAL_PATH_MAX 256
+#define HAL_PATH_MAX 512
 #define HAL_HELPER_TIMEOUT 10000
 
-int util_compute_time_remaining (const char *id, int chargeRate, int chargeLevel, int chargeLastFull, 
-				 gboolean isDischarging, gboolean isCharging, gboolean guessChargeRate);
-
-int util_compute_percentage_charge (const char *id, int chargeLevel, int chargeLastFull);
-
 gboolean hal_util_remove_trailing_slash (gchar *path);
 
 const gchar *hal_util_get_last_element (const gchar *s);
@@ -50,6 +53,10 @@ gboolean hal_util_get_int_from_file (const gchar *directory, const gchar *file,
 
 gboolean hal_util_set_int_from_file (HalDevice *d, const gchar *key, const gchar *directory, const gchar *file, gint base);
 
+gboolean hal_util_get_uint64_from_file (const gchar *directory, const gchar *file, guint64 *result, gint base);
+
+gboolean hal_util_set_uint64_from_file (HalDevice *d, const gchar *key, const gchar *directory, const gchar *file, gint base);
+
 gchar *hal_util_get_string_from_file (const gchar *directory, const gchar *file);
 
 gboolean hal_util_set_string_from_file (HalDevice *d, const gchar *key, const gchar *directory, const gchar *file);
@@ -97,4 +104,6 @@ gchar *hal_util_strdup_valid_utf8 (const char *str);
 
 void hal_util_hexdump (const void *buf, unsigned int size);
 
+gboolean hal_util_is_mounted_by_hald (const char *mount_point);
+
 #endif /* UTIL_H */
