@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: libhal-storage.c,v 1.32 2006/02/23 00:23:13 david Exp $
+ * CVSID: $Id$
  *
  * libhal-storage.c : HAL convenience library for storage devices and volumes
  *
@@ -57,17 +57,6 @@
 # define N_(String) (String)
 #endif
 
-/**
- * @defgroup LibHalStorage HAL Storage and Volumes library
- * @brief HAL Storage and Volumes library
- *
- *  @{
- */
-
-/**
- * @typedef typedef struct IconMappingEntry_s {} IconMappingEntry
- * @brief typdef of IconMappingEntry_s
- */
 typedef struct IconMappingEntry_s {
 	LibHalStoragePolicyIcon icon;
 	char *path;
@@ -188,7 +177,7 @@ libhal_volume_policy_compute_size_as_string (LibHalVolume *volume)
 					  ((double)size)/((double)cur), sizes_str[cur_str]);
 				result = strdup (buf);
 			} else {
-				snprintf (buf, MAX_STRING_SZ, "%lld%s", size / cur, sizes_str[cur_str]);
+				snprintf (buf, MAX_STRING_SZ, "%llu%s", (long long unsigned int) size / cur, sizes_str[cur_str]);
 				result = strdup (buf);
 				}
 			goto out;
@@ -289,7 +278,8 @@ libhal_drive_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *volu
 		}
 		if ((drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDRW) &&
 		    (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW)) {
-                        if(drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL)
+                        if(drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL || 
+			   drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRWDL)
                                 second = "/DVD±RW DL";
                         else
                                 second = "/DVD±RW";
@@ -697,6 +687,7 @@ struct LibHalDrive_s {
 	char *model;              /* may be "", is never NULL */
 	dbus_bool_t is_hotpluggable;
 	dbus_bool_t is_removable;
+	dbus_bool_t is_media_detected;
 	dbus_bool_t requires_eject;
 
 	LibHalDriveType type;
@@ -718,6 +709,10 @@ struct LibHalDrive_s {
 
 	dbus_bool_t no_partitions_hint;
 
+	dbus_uint64_t drive_size;
+	dbus_uint64_t drive_media_size;
+	char *partition_scheme;
+
 	LibHalContext *hal_ctx;
 
 	char **capabilities;
@@ -733,6 +728,7 @@ struct LibHalVolume_s {
 	char *device_file;
 	char *volume_label; /* may be NULL, is never "" */
 	dbus_bool_t is_mounted;
+	dbus_bool_t is_mounted_read_only; /* TRUE iff is_mounted and r/o fs */
 	char *mount_point;  /* NULL iff !is_mounted */
 	char *fstype;       /* NULL iff !is_mounted or unknown */
 	char *fsversion;
@@ -743,10 +739,16 @@ struct LibHalVolume_s {
 
 	dbus_bool_t is_partition;
 	unsigned int partition_number;
+	char *partition_scheme;
+	char *partition_type;
+	char *partition_label;
+	char *partition_uuid;
+	char **partition_flags;
 
 	int msdos_part_table_type;
+	dbus_uint64_t msdos_part_table_start;
+	dbus_uint64_t msdos_part_table_size;
 	
-
 	dbus_bool_t is_disc;
 	LibHalVolumeDiscType disc_type;
 	dbus_bool_t disc_has_audio;
@@ -767,6 +769,12 @@ struct LibHalVolume_s {
 	char *crypto_backing_volume;
 
 	char mount_options[MOUNT_OPTIONS_SIZE];
+
+	dbus_uint64_t volume_size;
+	dbus_uint64_t disc_capacity;
+
+	dbus_uint64_t partition_start_offset;
+	dbus_uint64_t partition_media_size;
 };
 
 const char *
@@ -797,11 +805,14 @@ libhal_drive_free (LibHalDrive *drive)
 	libhal_free_string (drive->model);
 	libhal_free_string (drive->type_textual);
 	libhal_free_string (drive->physical_device);
+	libhal_free_string (drive->dedicated_icon_drive);
+	libhal_free_string (drive->dedicated_icon_volume);
 	libhal_free_string (drive->serial);
 	libhal_free_string (drive->firmware_version);
 	libhal_free_string (drive->desired_mount_point);
 	libhal_free_string (drive->mount_filesystem);
 	libhal_free_string_array (drive->capabilities);
+	libhal_free_string (drive->partition_scheme);
 
 	free (drive);
 }
@@ -827,6 +838,13 @@ libhal_volume_free (LibHalVolume *vol)
 	libhal_free_string (vol->desired_mount_point);
 	libhal_free_string (vol->mount_filesystem);
 	libhal_free_string (vol->crypto_backing_volume);
+	libhal_free_string (vol->storage_device);
+
+	libhal_free_string (vol->partition_scheme);
+	libhal_free_string (vol->partition_type);
+	libhal_free_string (vol->partition_label);
+	libhal_free_string (vol->partition_uuid);
+	libhal_free_string_array (vol->partition_flags);
 
 	free (vol);
 }
@@ -859,6 +877,7 @@ out:
 #define LIBHAL_PROP_EXTRACT_BEGIN if (FALSE)
 #define LIBHAL_PROP_EXTRACT_END ;
 #define LIBHAL_PROP_EXTRACT_INT(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it)
+#define LIBHAL_PROP_EXTRACT_UINT64(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_UINT64) _where_ = libhal_psi_get_uint64 (&it)
 #define LIBHAL_PROP_EXTRACT_STRING(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRING) _where_ = (libhal_psi_get_string (&it) != NULL && strlen (libhal_psi_get_string (&it)) > 0) ? strdup (libhal_psi_get_string (&it)) : NULL
 #define LIBHAL_PROP_EXTRACT_BOOL(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ = libhal_psi_get_bool (&it)
 #define LIBHAL_PROP_EXTRACT_BOOL_BITFIELD(_property_, _where_, _field_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ |= libhal_psi_get_bool (&it) ? _field_ : 0
@@ -924,15 +943,19 @@ libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
 		LIBHAL_PROP_EXTRACT_STRING ("storage.vendor",            drive->vendor);
 		LIBHAL_PROP_EXTRACT_STRING ("storage.model",             drive->model);
 		LIBHAL_PROP_EXTRACT_STRING ("storage.drive_type",        drive->type_textual);
-
+		LIBHAL_PROP_EXTRACT_UINT64 ("storage.size", 		 drive->drive_size); 
 
 		LIBHAL_PROP_EXTRACT_STRING ("storage.icon.drive",        drive->dedicated_icon_drive);
 		LIBHAL_PROP_EXTRACT_STRING ("storage.icon.volume",       drive->dedicated_icon_volume);
 
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.hotpluggable",      drive->is_hotpluggable);
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.removable",         drive->is_removable);
+		LIBHAL_PROP_EXTRACT_BOOL   ("storage.removable.media_available", drive->is_media_detected);
+		LIBHAL_PROP_EXTRACT_UINT64 ("storage.removable.media_size", drive->drive_media_size); 
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.requires_eject",    drive->requires_eject);
 
+		LIBHAL_PROP_EXTRACT_STRING ("storage.partitioning_scheme", drive->partition_scheme); 
+
 		LIBHAL_PROP_EXTRACT_STRING ("storage.physical_device",   drive->physical_device);
 		LIBHAL_PROP_EXTRACT_STRING ("storage.firmware_version",  drive->firmware_version);
 		LIBHAL_PROP_EXTRACT_STRING ("storage.serial",            drive->serial);
@@ -942,6 +965,7 @@ libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvd", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDROM);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusr", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrw", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrwdl", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRWDL);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrdl", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdr", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDR);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdrw", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDRW);
@@ -1029,6 +1053,7 @@ libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
 	return drive;
 
 error:
+	LIBHAL_FREE_DBUS_ERROR(&error);
 	libhal_free_string (bus_textual);
 	libhal_free_property_set (properties);
 	libhal_drive_free (drive);
@@ -1102,7 +1127,19 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 
 		LIBHAL_PROP_EXTRACT_BEGIN;
 
+		LIBHAL_PROP_EXTRACT_BOOL   ("volume.is_partition",                    vol->is_partition);
+		LIBHAL_PROP_EXTRACT_INT    ("volume.partition.number",                vol->partition_number);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.partition.scheme",                vol->partition_scheme);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.partition.type",                  vol->partition_type);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.partition.label",                 vol->partition_label);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.partition.uuid",                  vol->partition_uuid);
+		LIBHAL_PROP_EXTRACT_STRLIST ("volume.partition.flags",                vol->partition_flags);
+
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.partition.start", 		      vol->partition_start_offset); 
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.partition.media_size",            vol->partition_media_size); 
 		LIBHAL_PROP_EXTRACT_INT    ("volume.partition.msdos_part_table_type", vol->msdos_part_table_type);
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.partition.msdos_part_table_start", vol->msdos_part_table_start);
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.partition.msdos_part_table_size", vol->msdos_part_table_size);
 
 		LIBHAL_PROP_EXTRACT_INT    ("block.minor",               vol->device_minor);
 		LIBHAL_PROP_EXTRACT_INT    ("block.major",               vol->device_major);
@@ -1114,11 +1151,13 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 
 		LIBHAL_PROP_EXTRACT_INT    ("volume.block_size",         vol->block_size);
 		LIBHAL_PROP_EXTRACT_INT    ("volume.num_blocks",         vol->num_blocks);
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.size", 		 vol->volume_size); 
 		LIBHAL_PROP_EXTRACT_STRING ("volume.label",              vol->volume_label);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.mount_point",        vol->mount_point);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.fstype",             vol->fstype);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.fsversion",             vol->fsversion);
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.is_mounted",         vol->is_mounted);
+		LIBHAL_PROP_EXTRACT_BOOL   ("volume.is_mounted_read_only", vol->is_mounted_read_only);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.fsusage",            vol_fsusage_textual);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.uuid",               vol->uuid);
 
@@ -1131,6 +1170,7 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.disc.is_appendable", vol->disc_is_appendable);
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.disc.is_blank",      vol->disc_is_blank);
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.disc.is_rewritable", vol->disc_is_rewritable);
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.disc.capacity",      vol->disc_capacity);
 
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.policy.should_mount",        vol->should_mount);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.policy.desired_mount_point", vol->desired_mount_point);
@@ -1185,6 +1225,8 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 			vol->fsusage = LIBHAL_VOLUME_USAGE_RAID_MEMBER;
 		} else if (strcmp (vol_fsusage_textual, "crypto") == 0) {
 			vol->fsusage = LIBHAL_VOLUME_USAGE_CRYPTO;
+		} else if (strcmp (vol_fsusage_textual, "other") == 0) {
+			vol->fsusage = LIBHAL_VOLUME_USAGE_OTHER;
 		} else {
 			vol->fsusage = LIBHAL_VOLUME_USAGE_UNKNOWN;
 		} 
@@ -1195,6 +1237,9 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 	libhal_free_property_set (properties);
 	return vol;
 error:
+	if (dbus_error_is_set (&error)) {
+		dbus_error_free (&error);
+	}
 	libhal_free_string (vol_fsusage_textual);
 	libhal_free_string (disc_type_textual);
 	libhal_free_property_set (properties);
@@ -1217,6 +1262,34 @@ libhal_volume_get_msdos_part_table_type (LibHalVolume *volume)
 	return volume->msdos_part_table_type;
 }
 
+/** If the volume is on a drive with a MSDOS style partition table, return
+ *  the partition start offset according to the partition table.
+ *
+ *  @param  volume              Volume object
+ *  @return                     The partition start offset or -1 if volume isnt
+ *                              a partition or the media the volume stems from
+ *                              isn't partition with a MS DOS style table
+ */
+dbus_uint64_t
+libhal_volume_get_msdos_part_table_start (LibHalVolume *volume)
+{
+	return volume->msdos_part_table_start;
+}
+
+/** If the volume is on a drive with a MSDOS style partition table, return
+ *  the partition size according to the partition table.
+ *
+ *  @param  volume              Volume object
+ *  @return                     The partition size or -1 if volume is not
+ *                              a partition or the media the volume stems from
+ *                              isn't partition with a MS DOS style table
+ */
+dbus_uint64_t
+libhal_volume_get_msdos_part_table_size (LibHalVolume *volume)
+{
+	return volume->msdos_part_table_size;
+}
+
 /***********************************************************************/
 
 /** Get the drive object that either is (when given e.g. /dev/sdb) or contains
@@ -1243,8 +1316,10 @@ libhal_drive_from_device_file (LibHalContext *hal_ctx, const char *device_file)
 
 	dbus_error_init (&error);
 	if ((hal_udis = libhal_manager_find_device_string_match (hal_ctx, "block.device", 
-								 device_file, &num_hal_udis, &error)) == NULL)
+								 device_file, &num_hal_udis, &error)) == NULL) {
+		LIBHAL_FREE_DBUS_ERROR(&error);
 		goto out;
+	}
 
 	for (i = 0; i < num_hal_udis; i++) {
 		char *udi;
@@ -1266,6 +1341,8 @@ libhal_drive_from_device_file (LibHalContext *hal_ctx, const char *device_file)
 		} else if (libhal_device_query_capability (hal_ctx, udi, "storage", &err2)) {
 			found_udi = strdup (udi);
 		}
+		LIBHAL_FREE_DBUS_ERROR(&err1);
+		LIBHAL_FREE_DBUS_ERROR(&err2);
 	}
 
 	libhal_free_string_array (hal_udis);
@@ -1321,13 +1398,23 @@ libhal_volume_from_device_file (LibHalContext *hal_ctx, const char *device_file)
 
 	free (found_udi);
 out:
+	LIBHAL_FREE_DBUS_ERROR(&error);
 	return result;
 }
 
 dbus_uint64_t
 libhal_volume_get_size (LibHalVolume *volume)
 {
-	return ((dbus_uint64_t)volume->block_size) * ((dbus_uint64_t)volume->num_blocks);
+	if (volume->volume_size > 0)
+		return volume->volume_size;
+	else
+		return ((dbus_uint64_t)volume->block_size) * ((dbus_uint64_t)volume->num_blocks);
+}
+
+dbus_uint64_t
+libhal_volume_get_disc_capacity (LibHalVolume *volume)
+{
+	return volume->disc_capacity;
 }
 
 
@@ -1343,6 +1430,31 @@ libhal_drive_uses_removable_media (LibHalDrive *drive)
 	return drive->is_removable;
 }
 
+dbus_bool_t
+libhal_drive_is_media_detected (LibHalDrive *drive)
+{
+	return drive->is_media_detected;
+}
+
+dbus_uint64_t
+libhal_drive_get_size (LibHalDrive *drive)
+{
+	return drive->drive_size;
+}
+
+dbus_uint64_t
+libhal_drive_get_media_size (LibHalDrive *drive)
+{
+	return drive->drive_media_size;
+}
+
+const char *
+libhal_drive_get_partition_scheme (LibHalDrive *drive)
+{
+	return drive->partition_scheme;
+}
+
+
 LibHalDriveType
 libhal_drive_get_type (LibHalDrive *drive)
 {
@@ -1463,6 +1575,12 @@ libhal_volume_is_mounted (LibHalVolume *volume)
 	return volume->is_mounted;
 }
 
+dbus_bool_t 
+libhal_volume_is_mounted_read_only (LibHalVolume *volume)
+{
+	return volume->is_mounted_read_only;
+}
+
 dbus_bool_t 
 libhal_volume_is_partition (LibHalVolume *volume)
 {
@@ -1481,6 +1599,49 @@ libhal_volume_get_partition_number (LibHalVolume *volume)
 	return volume->partition_number;
 }
 
+const char *
+libhal_volume_get_partition_scheme (LibHalVolume *volume)
+{
+	return volume->partition_scheme;
+}
+
+const char *
+libhal_volume_get_partition_type (LibHalVolume *volume)
+{
+	return volume->partition_type;
+}
+
+const char *
+libhal_volume_get_partition_label (LibHalVolume *volume)
+{
+	return volume->partition_label;
+}
+
+const char *
+libhal_volume_get_partition_uuid (LibHalVolume *volume)
+{
+	return volume->partition_uuid;
+}
+
+const char **
+libhal_volume_get_partition_flags (LibHalVolume *volume)
+{
+	return (const char **) volume->partition_flags;
+}
+
+
+dbus_uint64_t 
+libhal_volume_get_partition_start_offset (LibHalVolume *volume)
+{
+	return volume->partition_start_offset;
+}
+
+dbus_uint64_t
+libhal_volume_get_partition_media_size (LibHalVolume *volume)
+{
+	return volume->partition_media_size;
+}
+
 const char *
 libhal_volume_get_label (LibHalVolume *volume)
 {
@@ -1564,8 +1725,10 @@ libhal_drive_find_all_volumes (LibHalContext *hal_ctx, LibHalDrive *drive, int *
 	/* get initial list... */
 	dbus_error_init (&error);
 	if ((udis = libhal_manager_find_device_string_match (hal_ctx, "block.storage_device", 
-							     drive_udi, &num_udis, &error)) == NULL)
+							     drive_udi, &num_udis, &error)) == NULL) {
+		LIBHAL_FREE_DBUS_ERROR(&error);
 		goto out;
+	}
 
 	result = malloc (sizeof (char *) * num_udis);
 	if (result == NULL)
@@ -1627,49 +1790,65 @@ libhal_volume_crypto_get_clear_volume_udi (LibHalContext *hal_ctx, LibHalVolume
 char *
 libhal_drive_policy_default_get_mount_root (LibHalContext *hal_ctx)
 {
+	char *result;
 	DBusError error;
 
 	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
 
 	dbus_error_init (&error);
-	return libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
-						  "storage.policy.default.mount_root", &error);
+	if ((result = libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
+						    "storage.policy.default.mount_root", &error)) == NULL) 
+		LIBHAL_FREE_DBUS_ERROR(&error);
+
+	return result;
 }
 
 dbus_bool_t
 libhal_drive_policy_default_use_managed_keyword (LibHalContext *hal_ctx)
 {
+	dbus_bool_t result;
 	DBusError error;
 
 	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, FALSE);
 
 	dbus_error_init (&error);
-	return libhal_device_get_property_bool (hal_ctx, "/org/freedesktop/Hal/devices/computer",
-						"storage.policy.default.use_managed_keyword", &error);
+	if ((result = libhal_device_get_property_bool (hal_ctx, "/org/freedesktop/Hal/devices/computer",
+						  "storage.policy.default.use_managed_keyword", &error)) == FALSE)
+		LIBHAL_FREE_DBUS_ERROR(&error);
+
+	return result;
 }
 
 char *
 libhal_drive_policy_default_get_managed_keyword_primary (LibHalContext *hal_ctx)
 {
+	char *result;
 	DBusError error;
 
 	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
 
 	dbus_error_init (&error);
-	return libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
-						  "storage.policy.default.managed_keyword.primary", &error);
+	if ((result = libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
+						    "storage.policy.default.managed_keyword.primary", &error)) == NULL)
+		LIBHAL_FREE_DBUS_ERROR(&error);
+
+	return result;
 }
 
 char *
 libhal_drive_policy_default_get_managed_keyword_secondary (LibHalContext *hal_ctx)
 {
+	char *result;
 	DBusError error;
 
 	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
 
 	dbus_error_init (&error);
-	return libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
-						  "storage.policy.default.managed_keyword.secondary", &error);
+	if ((result = libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
+						    "storage.policy.default.managed_keyword.secondary", &error)) == NULL)
+		LIBHAL_FREE_DBUS_ERROR(&error);
+
+	return result;
 }
 
 /*************************************************************************/
@@ -1712,8 +1891,11 @@ mopts_collect (LibHalContext *hal_ctx, const char *namespace, int namespace_len,
 
 	/* first collect from root computer device */
 	properties = libhal_device_get_all_properties (hal_ctx, udi, &error);
-	if (properties == NULL)
-		goto error;
+	if (properties == NULL ) {
+		LIBHAL_FREE_DBUS_ERROR(&error);
+		return;
+	}
+
 	for (libhal_psi_init (&it, properties); libhal_psi_has_more (&it); libhal_psi_next (&it)) {
 		int type;
 		char *key;
@@ -1767,7 +1949,7 @@ mopts_collect (LibHalContext *hal_ctx, const char *namespace, int namespace_len,
 			}
 		}
 	}
-error:
+	
 	libhal_free_property_set (properties);
 }
 
