@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: libhal.h,v 1.32 2006/02/12 21:47:28 david Exp $
+ * CVSID: $Id$
  *
  * libhal.h : HAL daemon C convenience library headers
  *
@@ -35,8 +35,33 @@ extern "C" {
 #endif
 #endif
 
-/** Checks if LibHalContext *ctx == NULL */
-#define LIBHAL_CHECK_LIBHALCONTEXT(_ctx_, _ret_)					\
+#if defined(__GNUC__)
+#define LIBHAL_DEPRECATED __attribute__ ((deprecated))  
+#else  
+#define LIBHAL_DEPRECATED  
+#endif 
+
+
+#define LIBHAL_FREE_DBUS_ERROR(_dbus_error_)					\
+	do {									\
+		if (dbus_error_is_set(_dbus_error_))				\
+			dbus_error_free (_dbus_error_);				\
+		else								\
+			fprintf (stderr,					\
+				 "%s %d : INFO: called LIBHAL_FREE_DBUS_ERROR "	\
+				 "but dbusError was not set.\n", 		\
+				 __FILE__, __LINE__);				\
+	} while (0)
+
+
+/**
+ * LIBHAL_CHECK_LIBHALCONTEXT:
+ * @_ctx_: the context
+ * @_ret_: what to use for return value if context is invalid
+ *
+ * Handy macro for checking whether a context is valid.
+ */
+#define LIBHAL_CHECK_LIBHALCONTEXT(_ctx_, _ret_)				\
 	do {									\
 		if (_ctx_ == NULL) {						\
 			fprintf (stderr,					\
@@ -46,16 +71,14 @@ extern "C" {
 		}								\
 	} while(0)
 
-/**
- * @addtogroup LibHal
+/** 
+ * LibHalPropertyType:
  *
- * @{
+ * Possible types for properties on hal device objects 
  */
-
-/** Possible types for properties on hal device objects */
 typedef enum {
         /** Used to report error condition */
-	LIBHAL_PROPERTY_TYPE_INVALID     =    DBUS_TYPE_INVALID,
+	LIBHAL_PROPERTY_TYPE_INVALID =    DBUS_TYPE_INVALID,
 
 	/** Type for 32-bit signed integer property */
 	LIBHAL_PROPERTY_TYPE_INT32   =    DBUS_TYPE_INT32,
@@ -73,65 +96,78 @@ typedef enum {
 	LIBHAL_PROPERTY_TYPE_STRING  =    DBUS_TYPE_STRING,
 
 	/** Type for list of UTF-8 strings property */
-	LIBHAL_PROPERTY_TYPE_STRLIST =     ((int) (DBUS_TYPE_STRING<<8)+('l'))
+	LIBHAL_PROPERTY_TYPE_STRLIST =    ((int) (DBUS_TYPE_STRING<<8)+('l'))
 } LibHalPropertyType;
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-typedef struct LibHalContext_s LibHalContext;
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
-/** Type for function in application code that integrates a DBusConnection 
- *  object into its own mainloop.
+typedef struct LibHalContext_s LibHalContext;
+
+/** 
+ * LibHalIntegrateDBusIntoMainLoop:
+ * @ctx: context for connection to hald
+ * @dbus_connection: DBus connection to use in ctx
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  dbus_connection     DBus connection to use in ctx
+ * Type for function in application code that integrates a
+ * DBusConnection object into its own mainloop.
  */
 typedef void (*LibHalIntegrateDBusIntoMainLoop) (LibHalContext *ctx,
 						 DBusConnection *dbus_connection);
 
-/** Type for callback when a device is added.
+/** 
+ * LibHalDeviceAdded:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  udi                 Unique Device Id
+ * Type for callback when a device is added.
  */
 typedef void (*LibHalDeviceAdded) (LibHalContext *ctx, 
 				   const char *udi);
 
-/** Type for callback when a device is removed. 
+/** 
+ * LibHalDeviceRemoved:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  udi                 Unique Device Id
+ * Type for callback when a device is removed. 
  */
 typedef void (*LibHalDeviceRemoved) (LibHalContext *ctx, 
 				     const char *udi);
 
-/** Type for callback when a device gains a new capability
+/** 
+ * LibHalDeviceNewCapability:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
+ * @capability: capability of the device
+ *
+ * Type for callback when a device gains a new capability.
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  udi                 Unique Device Id
- *  @param  capability          Capability of the device
  */
 typedef void (*LibHalDeviceNewCapability) (LibHalContext *ctx, 
 					   const char *udi,
 					   const char *capability);
 
-/** Type for callback when a device loses a capability
+/** 
+ * LibHalDeviceLostCapability:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
+ * @capability: capability of the device
+ *
+ * Type for callback when a device loses a capability.
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  udi                 Unique Device Id
- *  @param  capability          Capability of the device
  */
 typedef void (*LibHalDeviceLostCapability) (LibHalContext *ctx, 
 					    const char *udi,
 					    const char *capability);
 
-/** Type for callback when a property of a device changes. 
+/** 
+ * LibHalDevicePropertyModified:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
+ * @key: name of the property that has changed
+ * @is_removed: whether or not property was removed
+ * @is_added: whether or not property was added
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  udi                 Unique Device Id
- *  @param  key                 Name of the property that has changed
- *  @param  is_removed          Whether or not property was removed
- *  @param  is_added            Whether or not property was added
+ * Type for callback when a property of a device changes. 
  */
 typedef void (*LibHalDevicePropertyModified) (LibHalContext *ctx,
 					      const char *udi,
@@ -139,14 +175,14 @@ typedef void (*LibHalDevicePropertyModified) (LibHalContext *ctx,
 					      dbus_bool_t is_removed,
 					      dbus_bool_t is_added);
 
-/** Type for callback when a non-continuous condition occurs on a device
+/** 
+ * LibHalDeviceCondition:
+ * @ctx: context for connection to hald
+ * @udi: the Unique Device Id
+ * @condition_name: name of the condition, e.g. ProcessorOverheating. Consult the HAL spec for details
+ * @condition_detail: detail of condition
  *
- *  @param  ctx                 Context for connection to hald
- *  @param  udi                 Unique Device Id
- *  @param  condition_name      Name of the condition, e.g. 
- *                              ProcessorOverheating. Consult the HAL spec
- *                              for possible conditions
- *  @param  condition_detail    User-readable details of condition
+ * Type for callback when a non-continuous condition occurs on a device.
  */
 typedef void (*LibHalDeviceCondition) (LibHalContext *ctx,
 				       const char *udi,
@@ -306,7 +342,7 @@ dbus_bool_t libhal_device_property_strlist_prepend (LibHalContext *ctx,
 dbus_bool_t libhal_device_property_strlist_remove_index (LibHalContext *ctx, 
 							 const char *udi,
 							 const char *key,
-							 unsigned int index,
+							 unsigned int idx,
 							 DBusError *error);
 
 /* Remove a specified string from a property of type strlist. */
@@ -328,14 +364,48 @@ LibHalPropertyType libhal_device_get_property_type (LibHalContext *ctx,
 						    const char *key,
 						    DBusError *error);
 
+struct LibHalChangeSet_s;
+typedef struct LibHalChangeSet_s LibHalChangeSet;
+
+LibHalChangeSet *libhal_device_new_changeset (const char *udi);
+
+dbus_bool_t libhal_changeset_set_property_string (LibHalChangeSet *changeset,
+						  const char *key,
+						  const char *value);
+
+dbus_bool_t libhal_changeset_set_property_int (LibHalChangeSet *changeset,
+					       const char *key,
+					       dbus_int32_t value);
+
+dbus_bool_t libhal_changeset_set_property_uint64 (LibHalChangeSet *changeset,
+						  const char *key,
+						  dbus_uint64_t value);
+
+dbus_bool_t libhal_changeset_set_property_double (LibHalChangeSet *changeset,
+						  const char *key,
+						  double value);
+
+dbus_bool_t libhal_changeset_set_property_bool (LibHalChangeSet *changeset,
+						const char *key,
+						dbus_bool_t value);
+
+dbus_bool_t libhal_changeset_set_property_strlist (LibHalChangeSet *changeset,
+						   const char *key,
+						   const char **value);
+
+dbus_bool_t libhal_device_commit_changeset (LibHalContext *ctx,
+					    LibHalChangeSet *changeset,
+					    DBusError *error);
+
+void libhal_device_free_changeset (LibHalChangeSet *changeset);
+
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
 struct LibHalProperty_s;
 typedef struct LibHalProperty_s LibHalProperty;
 
 struct LibHalPropertySet_s;
 typedef struct LibHalPropertySet_s LibHalPropertySet;
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
+
 
 /* Retrieve all the properties on a device. */
 LibHalPropertySet *libhal_device_get_all_properties (LibHalContext *ctx, 
@@ -348,18 +418,22 @@ void libhal_free_property_set (LibHalPropertySet *set);
 /* Get the number of properties in a property set. */
 unsigned int libhal_property_set_get_num_elems (LibHalPropertySet *set);
 
-/** Iterator for inspecting all properties */
+/** 
+ * LibHalPropertySetIterator: 
+ * 
+ * Iterator for inspecting all properties. Do not access any members;
+ * use the libhal_psi_* family of functions instead.
+ */
 struct LibHalPropertySetIterator_s {
 	LibHalPropertySet *set;    /**< Property set we are iterating over */
-	unsigned int index;        /**< Index into current element */
+	unsigned int idx;          /**< Index into current element */
 	LibHalProperty *cur_prop;  /**< Current property being visited */
 	void *reservered0;         /**< Reserved for future use */
 	void *reservered1;         /**< Reserved for future use */
 };
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 typedef struct LibHalPropertySetIterator_s LibHalPropertySetIterator;
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
 /* Initialize a property set iterator. */
 void libhal_psi_init (LibHalPropertySetIterator *iter, LibHalPropertySet *set);
@@ -505,14 +579,23 @@ dbus_bool_t libhal_device_reprobe (LibHalContext *ctx,
 				   const char *udi,
 				   DBusError *error);
 
-/* Emit a condition from a device */
+/* Emit a condition from a device (for hald helpers only) */
 dbus_bool_t libhal_device_emit_condition (LibHalContext *ctx,
 					  const char *udi,
 					  const char *condition_name,
 					  const char *condition_details,
 					  DBusError *error);
 
-/** @} */
+/* Claim an interface for a device (for hald helpers only) */
+dbus_bool_t libhal_device_claim_interface (LibHalContext *ctx,
+					   const char *udi,
+					   const char *interface_name,
+					   const char *introspection_xml,
+					   DBusError *error);
+
+/* hald waits for all addons to call this function before announcing the addon (for hald helpers only) */
+dbus_bool_t libhal_device_addon_is_ready (LibHalContext *ctx, const char *udi, DBusError *error);
+
 
 #if defined(__cplusplus)
 }
