@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: hal-device.c,v 1.4 2005/11/02 15:38:14 david Exp $
+ * CVSID: $Id$
  *
  * hal-device.c : add devices HAL
  *
@@ -45,7 +45,7 @@
 #endif
 
 #include <dbus/dbus.h>
-#include <libhal/libhal.h>
+#include <libhal.h>
 
 typedef struct {
 	char *udi;
@@ -63,7 +63,7 @@ typedef struct lh_prop_s {
 		double double_value;
 		dbus_bool_t bool_value;
 		char **strlist_value;
-	};
+	} v;
 } lh_prop_t;
 
 
@@ -93,7 +93,7 @@ struct option options[] = {
 	{ "remove", 1, NULL, 'r' },
 	{ "add", 1, NULL, 'a' },
 	{ "help", 0, NULL, 'h' },
-	{}
+	{ 0, 0, 0, 0 }
 };
 
 
@@ -132,6 +132,7 @@ int main(int argc, char **argv)
 
 	if (!(conn = dbus_bus_get(DBUS_BUS_SYSTEM, &error))) {
 		fprintf(stderr, "error: dbus_bus_get: %s: %s\n", error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 2;
 	}
 
@@ -140,6 +141,7 @@ int main(int argc, char **argv)
 	if (!libhal_ctx_set_dbus_connection(hal_ctx, conn)) return 4;
 	if (!libhal_ctx_init(hal_ctx, &error)) {
 		fprintf(stderr, "error: libhal_ctx_init: %s: %s\n", error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 5;
 	}
 
@@ -155,7 +157,7 @@ int main(int argc, char **argv)
 
 	libhal_ctx_shutdown(hal_ctx, &error);
 	libhal_ctx_free(hal_ctx);
-	dbus_connection_disconnect(conn);
+	dbus_connection_close(conn);
 	dbus_connection_unref(conn);
 	dbus_error_free(&error);
 
@@ -192,7 +194,13 @@ int dump_devices(LibHalContext *hal_ctx, char *arg)
 		if (*arg == '/') {
 			udi = arg;
 		} else {
+#ifdef HAVE_ASPRINTF
 			asprintf(&udi, "/org/freedesktop/Hal/devices/%s", arg);
+#else
+			udi = calloc(1, sizeof ("/org/freedesktop/Hal/devices/%s") + strlen(arg));
+			sprintf(udi, "/org/freedesktop/Hal/devices/%s", arg);
+
+#endif
 		}
 	}
 
@@ -201,6 +209,7 @@ int dump_devices(LibHalContext *hal_ctx, char *arg)
 	if (!udi) {
 		if (!(device_names = libhal_get_all_devices(hal_ctx, &num_devices, &error))) {
 			fprintf(stderr, "Empty HAL device list.\n");
+			LIBHAL_FREE_DBUS_ERROR (&error);
 			return 31;
 		}
 	} else {
@@ -299,7 +308,13 @@ int remove_udi(LibHalContext *hal_ctx, char *arg)
 	if (*arg == '/') {
 		udi = arg;
 	} else {
+#ifdef HAVE_ASPRINTF
 		asprintf(&udi, "/org/freedesktop/Hal/devices/%s", arg);
+#else
+		udi = calloc(1, sizeof ("/org/freedesktop/Hal/devices/%s") + strlen(arg));
+		sprintf(udi, "/org/freedesktop/Hal/devices/%s", arg);
+#endif
+
 	}
 
 	dbus_error_init(&error);
@@ -307,6 +322,7 @@ int remove_udi(LibHalContext *hal_ctx, char *arg)
 	if (opt.remove) {
 		if (!libhal_remove_device(hal_ctx, udi, &error)) {
 			fprintf(stderr, "%s: %s\n", error.name, error.message);
+			LIBHAL_FREE_DBUS_ERROR (&error);
 			return 12;
 		}
 
@@ -332,7 +348,12 @@ int add_udi(LibHalContext *hal_ctx, char *arg)
 	if (*arg == '/') {
 		udi = arg;
 	} else {
+#ifdef HAVE_ASPRINTF
 		asprintf(&udi, "/org/freedesktop/Hal/devices/%s", arg);
+#else
+		udi = calloc(1, sizeof ("/org/freedesktop/Hal/devices/%s") + strlen(arg));
+		sprintf(udi, "/org/freedesktop/Hal/devices/%s", arg);
+#endif
 	}
 
 	if (udi)
@@ -350,6 +371,7 @@ int add_udi(LibHalContext *hal_ctx, char *arg)
 
 		if (!new_dev.real_udi) {
 			fprintf(stderr, "%s: %s\n", error.name, error.message);
+			LIBHAL_FREE_DBUS_ERROR (&error);
 			free(new_dev.real_udi);
 
 			return 22;
@@ -371,9 +393,10 @@ int add_udi(LibHalContext *hal_ctx, char *arg)
 	if (!dev_exists) {
 		if (!libhal_device_commit_to_gdl(hal_ctx, new_dev.real_udi, new_dev.udi, &error)) {
 			fprintf(stderr, "%s: %s\n", error.name, error.message);
+			LIBHAL_FREE_DBUS_ERROR (&error);
 			free(new_dev.real_udi);
 
-			err = err ?: 23;
+			err = err ? err : 23;
 		}
 	}
 
@@ -463,44 +486,44 @@ void process_property(LibHalContext *hal_ctx, char *buf, lh_prop_t **prop)
 	if (*s == '\'') {
 		s_val = s + 1;
 		s = strrchr(s_val, '\'');
-		*(s ?: s_val) = 0;
+		*(s ? s : s_val) = 0;
 		p->type = LIBHAL_PROPERTY_TYPE_STRING;
-		p->str_value = strdup(s_val);
+		p->v.str_value = strdup(s_val);
 	} else if (*s == '{') {
 		s_val = s + 1;
 		s1 = strrchr(s_val, '}');
 		if (s1) *s1 = 0;
 		p->type = LIBHAL_PROPERTY_TYPE_STRLIST;
 		len = 0;
-		p->strlist_value = calloc(1, sizeof *p->strlist_value);
+		p->v.strlist_value = calloc(1, sizeof *p->v.strlist_value);
 		while (*s_val++ == '\'') {
 			s = skip_nonquote(s_val);
 			if (*s) *s++ = 0;
-			p->strlist_value = realloc(p->strlist_value, (len + 2) * sizeof *p->strlist_value);
-			p->strlist_value[len] = strdup(s_val);
-			p->strlist_value[++len] = NULL;
+			p->v.strlist_value = realloc(p->v.strlist_value, (len + 2) * sizeof *p->v.strlist_value);
+			p->v.strlist_value[len] = strdup(s_val);
+			p->v.strlist_value[++len] = NULL;
 			s_val = skip_nonquote(s);
 		}
 	} else if (!strncmp(s, "true", 4)) {
 		s += 4;
 		p->type = LIBHAL_PROPERTY_TYPE_BOOLEAN;
-		p->bool_value = TRUE;
+		p->v.bool_value = TRUE;
 	} else if (!strncmp(s, "false", 5)) {
 		s += 5;
 		p->type = LIBHAL_PROPERTY_TYPE_BOOLEAN;
-		p->bool_value = FALSE;
+		p->v.bool_value = FALSE;
 	} else if ((s1 = skip_number(s)) != s) {
 		if (strstr(s1, "(int)")) {
 			*s1++ = 0;
 			p->type = LIBHAL_PROPERTY_TYPE_INT32;
-			p->int_value = strtol(s, NULL, 10);;
+			p->v.int_value = strtol(s, NULL, 10);
 		} else if (strstr(s1, "(uint64)")) {
 			*s1++ = 0;
 			p->type = LIBHAL_PROPERTY_TYPE_UINT64;
-			p->uint64_value = strtoull(s, NULL, 10);
+			p->v.uint64_value = strtoull(s, NULL, 10);
 		} else if (strstr(s1, "(double)")) {
 			p->type = LIBHAL_PROPERTY_TYPE_DOUBLE;
-			p->double_value = strtod(s, NULL);
+			p->v.double_value = strtod(s, NULL);
 		}
 
 		s = s1;
@@ -527,7 +550,7 @@ int add_properties(LibHalContext *hal_ctx, new_dev_t *nd, lh_prop_t *prop)
 
 	for(p = prop; p; p = p->next) {
 		if (!strcmp(p->key, "udi") && p->type == LIBHAL_PROPERTY_TYPE_STRING) {
-			udi2 = p->str_value;
+			udi2 = p->v.str_value;
 			continue;
 		}
 
@@ -538,6 +561,7 @@ int add_properties(LibHalContext *hal_ctx, new_dev_t *nd, lh_prop_t *prop)
 		    ( p->type != old_type || p->type == LIBHAL_PROPERTY_TYPE_STRLIST)) {
 			if (!libhal_device_remove_property(hal_ctx, nd->real_udi, p->key, &error)) {
 				fprintf(stderr, "%s: %s\n", error.name, error.message);
+				LIBHAL_FREE_DBUS_ERROR (&error);
 				return 41;
 			}
 		}
@@ -546,40 +570,46 @@ int add_properties(LibHalContext *hal_ctx, new_dev_t *nd, lh_prop_t *prop)
 			case LIBHAL_PROPERTY_TYPE_INVALID:
 				break;
 			case LIBHAL_PROPERTY_TYPE_BOOLEAN:
-				if (!libhal_device_set_property_bool(hal_ctx, nd->real_udi, p->key, p->bool_value, &error)) {
+				if (!libhal_device_set_property_bool(hal_ctx, nd->real_udi, p->key, p->v.bool_value, &error)) {
 					fprintf(stderr, "%s: %s\n", error.name, error.message);
+					LIBHAL_FREE_DBUS_ERROR (&error);
 					return 42;
 				}
 				break;
 			case LIBHAL_PROPERTY_TYPE_INT32:
-				if (!libhal_device_set_property_int(hal_ctx, nd->real_udi, p->key, p->int_value, &error)) {
+				if (!libhal_device_set_property_int(hal_ctx, nd->real_udi, p->key, p->v.int_value, &error)) {
 					fprintf(stderr, "%s: %s\n", error.name, error.message);
+					LIBHAL_FREE_DBUS_ERROR (&error);
 					return 42;
 				}
 				break;
 			case LIBHAL_PROPERTY_TYPE_UINT64:
-				if (!libhal_device_set_property_uint64(hal_ctx, nd->real_udi, p->key, p->uint64_value, &error)) {
+				if (!libhal_device_set_property_uint64(hal_ctx, nd->real_udi, p->key, p->v.uint64_value, &error)) {
 					fprintf(stderr, "%s: %s\n", error.name, error.message);
+					LIBHAL_FREE_DBUS_ERROR (&error);
 					return 42;
 				}
 				break;
 			case LIBHAL_PROPERTY_TYPE_DOUBLE:
-				if (!libhal_device_set_property_double(hal_ctx, nd->real_udi, p->key, p->double_value, &error)) {
+				if (!libhal_device_set_property_double(hal_ctx, nd->real_udi, p->key, p->v.double_value, &error)) {
 					fprintf(stderr, "%s: %s\n", error.name, error.message);
+					LIBHAL_FREE_DBUS_ERROR (&error);
 					return 42;
 				}
 				break;
 			case LIBHAL_PROPERTY_TYPE_STRING:
-				if (!strcmp(p->key, "info.udi")) udi3 = p->str_value;
-				if (!libhal_device_set_property_string(hal_ctx, nd->real_udi, p->key, p->str_value, &error)) {
+				if (!strcmp(p->key, "info.udi")) udi3 = p->v.str_value;
+				if (!libhal_device_set_property_string(hal_ctx, nd->real_udi, p->key, p->v.str_value, &error)) {
 					fprintf(stderr, "%s: %s\n", error.name, error.message);
+					LIBHAL_FREE_DBUS_ERROR (&error);
 					return 42;
 				}
 				break;
 			case LIBHAL_PROPERTY_TYPE_STRLIST:
-				for(s = p->strlist_value; *s; s++) {
+				for(s = p->v.strlist_value; *s; s++) {
 					if (!libhal_device_property_strlist_append(hal_ctx, nd->real_udi, p->key, *s, &error)) {
 						fprintf(stderr, "%s: %s\n", error.name, error.message);
+						LIBHAL_FREE_DBUS_ERROR (&error);
 						return 42;
 					}
 				}
@@ -606,10 +636,10 @@ lh_prop_t *free_properties(lh_prop_t *prop)
 		next = prop->next;
 
 		free(prop->key);
-		if (prop->type == LIBHAL_PROPERTY_TYPE_STRING) free(prop->str_value);
-		if (prop->type == LIBHAL_PROPERTY_TYPE_STRLIST && prop->strlist_value) {
-			for(s = prop->strlist_value; *s; ) free(*s++);
-			free(prop->strlist_value);
+		if (prop->type == LIBHAL_PROPERTY_TYPE_STRING) free(prop->v.str_value);
+		if (prop->type == LIBHAL_PROPERTY_TYPE_STRLIST && prop->v.strlist_value) {
+			for(s = prop->v.strlist_value; *s; ) free(*s++);
+			free(prop->v.strlist_value);
 		}
 		free(prop);
 	}
