@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: hal_find_by_property.c,v 1.3 2005/11/02 15:38:14 david Exp $
+ * CVSID: $Id$
  *
  * hal_find_by_property.c : Find hal devices
  *
@@ -33,7 +33,7 @@
 #include <unistd.h>
 #include <getopt.h>
 
-#include <libhal/libhal.h>
+#include <libhal.h>
 
 
 /** Print out program usage.
@@ -150,10 +150,12 @@ main (int argc, char *argv[])
 	}
 	if (!libhal_ctx_set_dbus_connection (hal_ctx, dbus_bus_get (DBUS_BUS_SYSTEM, &error))) {
 		fprintf (stderr, "error: libhal_ctx_set_dbus_connection: %s: %s\n", error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 1;
 	}
 	if (!libhal_ctx_init (hal_ctx, &error)) {
 		fprintf (stderr, "error: libhal_ctx_init: %s: %s\n", error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 1;
 	}
 
@@ -162,6 +164,7 @@ main (int argc, char *argv[])
 
 	if (dbus_error_is_set (&error)) {
 		fprintf (stderr, "error: %s: %s\n", error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 1;
 	}
 
