@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: lshal.c,v 1.19 2006/02/13 14:35:31 dkukawka Exp $
+ * CVSID: $Id$
  *
  * lshal.c : Show devices managed by HAL
  *
@@ -38,7 +38,11 @@
 #include <glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include <dbus/dbus-glib.h>
-#include <libhal/libhal.h>
+#include <libhal.h>
+
+#ifdef __SUNPRO_C
+#define __FUNCTION__ __func__
+#endif
 
 /**
  * @defgroup HalLsHal  List HAL devices
@@ -99,8 +103,10 @@ print_props (const char *udi)
 	 *        essence an IPC call and other stuff may
 	 *        be happening..
 	 */
-	if (props == NULL)
+	if (props == NULL) {
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return;
+	}
 
 	for (libhal_psi_init (&it, props); libhal_psi_has_more (&it); libhal_psi_next (&it)) {
 		type = libhal_psi_get_type (&it);
@@ -119,10 +125,10 @@ print_props (const char *udi)
 			break;
 
 		case LIBHAL_PROPERTY_TYPE_UINT64:
-			printf ("  %s = %lld  (0x%llx)  (uint64)\n",
+			printf ("  %s = %llu  (0x%llx)  (uint64)\n",
 				libhal_psi_get_key (&it),
-				libhal_psi_get_uint64 (&it),
-				libhal_psi_get_uint64 (&it));
+				(long long unsigned int) libhal_psi_get_uint64 (&it),
+				(long long unsigned int) libhal_psi_get_uint64 (&it));
 			break;
 
 		case LIBHAL_PROPERTY_TYPE_DOUBLE:
@@ -176,8 +182,10 @@ dump_device (const char *udi)
 
 	dbus_error_init (&error);
 
-	if (!libhal_device_exists (hal_ctx, udi, &error))
+	if (!libhal_device_exists (hal_ctx, udi, &error)) {
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return;
+	}
 
 	if (long_list) {
 		printf ("udi = '%s'\n", udi);
@@ -249,8 +257,10 @@ dump_devices (void)
 	dbus_error_init (&error);
 
 	device_names = libhal_get_all_devices (hal_ctx, &num_devices, &error);
-	if (device_names == NULL)
+	if (device_names == NULL) {
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		DIE (("Couldn't obtain list of devices\n"));
+	}
 
 	devices = malloc (sizeof(struct Device) * num_devices);
 	if (!devices) {
@@ -410,8 +420,8 @@ print_property (const char *udi, const char *key)
 	case LIBHAL_PROPERTY_TYPE_UINT64:
 		{
 			dbus_uint64_t value = libhal_device_get_property_uint64 (hal_ctx, udi, key, &error);
-			printf (long_list?"*** new value: %lld (0x%llx)  (uint64)\n":"%lld (0x%llx)",
-				value, value);
+			printf (long_list?"*** new value: %llu (0x%llx)  (uint64)\n":"%llu (0x%llx)",
+				(long long unsigned int) value, (long long unsigned int) value);
 		}
 		break;
 	case LIBHAL_PROPERTY_TYPE_DOUBLE:
@@ -450,6 +460,9 @@ print_property (const char *udi, const char *key)
 		fprintf (stderr, "Unknown type %d='%c'\n", type, type);
 		break;
 	}
+
+	if (dbus_error_is_set (&error))
+		dbus_error_free (&error);
 }
 
 /** Invoked when a property of a device in the Global Device List is
@@ -652,6 +665,7 @@ main (int argc, char *argv[])
 	if (conn == NULL) {
 		fprintf (stderr, "error: dbus_bus_get: %s: %s\n",
 			 error.name, error.message);
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 1;
 	}
 
@@ -672,6 +686,7 @@ main (int argc, char *argv[])
 			 error.name, error.message);
 		fprintf (stderr, "Could not initialise connection to hald. \n "
 				 "Normally this mean the HAL daemon (hald) is not running or not ready.\n");
+		LIBHAL_FREE_DBUS_ERROR (&error);
 		return 1;
 	}
 
@@ -692,16 +707,22 @@ main (int argc, char *argv[])
 		if( long_list || short_list || tree_view )
 			dump_devices ();
 		
+		if ( libhal_device_property_watch_all (hal_ctx, &error) == FALSE) {
+			fprintf (stderr, "error: monitoring devicelist - libhal_device_property_watch_all: %s: %s\n",
+				 error.name, error.message);
+			LIBHAL_FREE_DBUS_ERROR (&error);
+			return 1;
+		}
 		printf ("\nStart monitoring devicelist:\n"
 			"-------------------------------------------------\n");
-		libhal_device_property_watch_all (hal_ctx, &error);
 		g_main_loop_run (loop);
 	}
 
-	libhal_ctx_shutdown (hal_ctx, &error);
+	if ( libhal_ctx_shutdown (hal_ctx, &error) == FALSE)
+		LIBHAL_FREE_DBUS_ERROR (&error);
 	libhal_ctx_free (hal_ctx);
 
-	dbus_connection_disconnect (conn);
+	dbus_connection_close (conn);
 	dbus_connection_unref (conn);
 
 	if (show_device)
